<?php

namespace Bixo\Payment\Http\Controllers;

use Exception;
use Bixo\Payment\Actions\DetailWorkflow;
use Bixo\Payment\Http\Requests\DetailWorkflowRequest;
use Bixo\Payment\Http\Resources\DetailResource;
use Bixo\Payment\Models\Detail;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class DetailWorkflowController extends BaseController
{
    /**
     * Action controller function for detail.
     *
     * @param Model $detail
     * @param action next action for the detail.
     *
     * @return Response
     */
    public function __invoke(DetailWorkflowRequest $request, Detail $detail, $trasition)
    {
        try {
            $request = $request->all();
            $detail = DetailWorkflow::run($trasition, $detail, $request);
            $data = new DetailResource($detail);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('payment::detail.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('payment/detail/' . $detail->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('payment/detail/' . $detail->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the detail and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $detail
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $detail, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $detail = Detail::findBySignedId($detail);
        $transitions = $this->transitions($detail, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('payment::detail.name') . ' ' . trans('Approvals'))
            ->view('payment::public.detail.workflow')
            ->layout('mobile')
            ->data(compact('detail', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the detail.
     *
     * @param PublicRequest $request
     * @param string $detail
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $detail, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $detail = Detail::findBySignedId($detail);
        $user = user();
        $detail = DetailWorkflow::run($transition, $detail, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($detail, $user)
    {
        $transitions = [];
        foreach ($detail->workflow()->transitions($detail) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.detail.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'detail' => $detail->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $detail->workflow()->form($value);
            $array['label'] = trans('payment::detail.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
