<?php

namespace Bixo\Payment\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Payment\Forms\Invoice as InvoiceForm;
use Bixo\Payment\Http\Requests\InvoiceResourceRequest;
use Bixo\Payment\Http\Resources\InvoiceResource;
use Bixo\Payment\Http\Resources\InvoicesCollection;
use Bixo\Payment\Models\Invoice;
use Bixo\Payment\Actions\InvoiceAction;
use Bixo\Payment\Actions\InvoiceActions;


/**
 * Resource controller class for invoice.
 */
class InvoiceResourceController extends BaseController
{

    /**
     * Initialize invoice resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = InvoiceForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.payment.modules'), 'payment', guard_url('payment'));
            return $next($request);
        });
    }

    /**
     * Display a list of invoice.
     *
     * @return Response
     */
    public function index(InvoiceResourceRequest $request)
    {
        $request = $request->all();
        $page = InvoiceActions::run('paginate', $request);

        $data = new InvoicesCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('payment::invoice.names'))
            ->view('payment::invoice.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display invoice.
     *
     * @param Request $request
     * @param Model   $invoice
     *
     * @return Response
     */
    public function show(InvoiceResourceRequest $request, Invoice $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new InvoiceResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('payment::invoice.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('payment::invoice.show')
            ->output();
    }

    /**
     * Show the form for creating a new invoice.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(InvoiceResourceRequest $request, Invoice $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new InvoiceResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('payment::invoice.name'))
            ->view('payment::invoice.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new invoice.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(InvoiceResourceRequest $request, Invoice $model)
    {
        try {
            $request = $request->all();
            $model = InvoiceAction::run('store', $model, $request);
            $data = new InvoiceResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('payment::invoice.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('payment/invoice/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/payment/invoice'))
                ->redirect();
        }

    }

    /**
     * Show invoice for editing.
     *
     * @param Request $request
     * @param Model   $invoice
     *
     * @return Response
     */
    public function edit(InvoiceResourceRequest $request, Invoice $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new InvoiceResource($model);
        // return view('payment::invoice.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('payment::invoice.name'))
            ->view('payment::invoice.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the invoice.
     *
     * @param Request $request
     * @param Model   $invoice
     *
     * @return Response
     */
    public function update(InvoiceResourceRequest $request, Invoice $model)
    {
        try {
            $request = $request->all();
            $model = InvoiceAction::run('update', $model, $request);
            $data = new InvoiceResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('payment::invoice.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('payment/invoice/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('payment/invoice/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the invoice.
     *
     * @param Model   $invoice
     *
     * @return Response
     */
    public function destroy(InvoiceResourceRequest $request, Invoice $model)
    {
        try {

            $request = $request->all();
            $model = InvoiceAction::run('destroy', $model, $request);
            $data = new InvoiceResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('payment::invoice.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('payment/invoice/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('payment/invoice/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
