<?php

namespace Bixo\Payment\Http\Controllers;

use Exception;
use Bixo\Payment\Actions\InvoiceWorkflow;
use Bixo\Payment\Http\Requests\InvoiceWorkflowRequest;
use Bixo\Payment\Http\Resources\InvoiceResource;
use Bixo\Payment\Models\Invoice;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class InvoiceWorkflowController extends BaseController
{
    /**
     * Action controller function for invoice.
     *
     * @param Model $invoice
     * @param action next action for the invoice.
     *
     * @return Response
     */
    public function __invoke(InvoiceWorkflowRequest $request, Invoice $invoice, $trasition)
    {
        try {
            $request = $request->all();
            $invoice = InvoiceWorkflow::run($trasition, $invoice, $request);
            $data = new InvoiceResource($invoice);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('payment::invoice.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('payment/invoice/' . $invoice->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('payment/invoice/' . $invoice->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the invoice and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $invoice
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $invoice, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $invoice = Invoice::findBySignedId($invoice);
        $transitions = $this->transitions($invoice, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('payment::invoice.name') . ' ' . trans('Approvals'))
            ->view('payment::public.invoice.workflow')
            ->layout('mobile')
            ->data(compact('invoice', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the invoice.
     *
     * @param PublicRequest $request
     * @param string $invoice
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $invoice, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $invoice = Invoice::findBySignedId($invoice);
        $user = user();
        $invoice = InvoiceWorkflow::run($transition, $invoice, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($invoice, $user)
    {
        $transitions = [];
        foreach ($invoice->workflow()->transitions($invoice) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.invoice.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'invoice' => $invoice->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $invoice->workflow()->form($value);
            $array['label'] = trans('payment::invoice.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
